@@ -9,10 +9,8 @@
 # http://www.python.org/dev/peps/pep-0333/#the-server-gateway-side
 
 
-import os
-
 from ..pycompat import getattr
-from .. import pycompat
+from .. import encoding, pycompat
 
 from ..utils import procutil
 
@@ -23,7 +21,10 @@ def launch(application):
     procutil.setbinary(procutil.stdin)
     procutil.setbinary(procutil.stdout)
 
-    environ = dict(os.environ.items())  # re-exports
+    environ = {
+        k.decode('iso8859-1'): v.decode('iso8859-1')
+        for k, v in encoding.environ.items()
+    }  # re-exports
     environ.setdefault('PATH_INFO', '')
     if environ.get('SERVER_SOFTWARE', '').startswith('Microsoft-IIS'):
         # IIS includes script_name in PATH_INFO
