@@ -103,6 +103,7 @@ class ipeerconnection(interfaceutil.Interface):
     """
 
     ui = interfaceutil.Attribute("""ui.ui instance""")
+    path = interfaceutil.Attribute("""a urlutil.path instance or None""")
 
     def url():
         """Returns a URL string representing this peer.
@@ -123,12 +124,6 @@ class ipeerconnection(interfaceutil.Interface):
         can be used to interface with it. Otherwise returns ``None``.
         """
 
-    def peer():
-        """Returns an object conforming to this interface.
-
-        Most implementations will ``return self``.
-        """
-
     def canpush():
         """Returns a boolean indicating if this peer can be pushed to."""
 
@@ -181,6 +176,12 @@ class ipeercommands(interfaceutil.Interface):
         Returns a set of string capabilities.
         """
 
+    def get_cached_bundle_inline(path):
+        """Retrieve a clonebundle across the wire.
+
+        Returns a chunkbuffer
+        """
+
     def clonebundles():
         """Obtains the clone bundles manifest for the repo.
 
@@ -393,6 +394,10 @@ class peer:
 
     limitedarguments = False
 
+    def __init__(self, ui, path=None, remotehidden=False):
+        self.ui = ui
+        self.path = path
+
     def capable(self, name):
         caps = self.capabilities()
         if name in caps:
@@ -1405,6 +1410,14 @@ class imanifeststorage(interfaceutil.Interface):
         This one behaves the same way, except for manifest data.
         """
 
+    def get_revlog():
+        """return an actual revlog instance if any
+
+        This exist because a lot of code leverage the fact the underlying
+        storage is a revlog for optimization, so giving simple way to access
+        the revlog instance helps such code.
+        """
+
 
 class imanifestlog(interfaceutil.Interface):
     """Interface representing a collection of manifest snapshots.
@@ -1525,6 +1538,10 @@ class ilocalrepositorymain(interfaceutil.Interface):
         """Name of the repoview that is active on this repo."""
     )
 
+    vfs_map = interfaceutil.Attribute(
+        """a bytes-key → vfs mapping used by transaction and others"""
+    )
+
     wvfs = interfaceutil.Attribute(
         """VFS used to access the working directory."""
     )
@@ -1613,7 +1630,7 @@ class ilocalrepositorymain(interfaceutil.Interface):
     def close():
         """Close the handle on this repository."""
 
-    def peer():
+    def peer(path=None):
         """Obtain an object conforming to the ``peer`` interface."""
 
     def unfiltered():
@@ -1807,6 +1824,9 @@ class ilocalrepositorymain(interfaceutil.Interface):
     def lock(wait=True):
         """Lock the repository store and return a lock instance."""
 
+    def currentlock():
+        """Return the lock if it's held or None."""
+
     def wlock(wait=True):
         """Lock the non-store parts of the repository."""
 
