@@ -36,7 +36,7 @@ def _findupdatetarget(repo, nodes):
     currentbranch = repo[None].branch()
 
     if (
-        util.safehasattr(repo, b'mq')
+        util.safehasattr(repo, 'mq')
         and p2 != repo.nullid
         and p2 in [x.node for x in repo.mq.applied]
     ):
