@@ -11,10 +11,7 @@ import stat
 from ..i18n import _
 from ..pycompat import getattr
 from .. import (
-    changelog,
     error,
-    filelog,
-    manifest,
     metadata,
     pycompat,
     requirements,
@@ -47,32 +44,7 @@ def get_sidedata_helpers(srcrepo, dstrepo):
     return sidedatamod.get_sidedata_helpers(srcrepo, dstrepo._wanted_sidedata)
 
 
-def _revlogfrompath(repo, rl_type, path):
-    """Obtain a revlog from a repo path.
-
-    An instance of the appropriate class is returned.
-    """
-    if rl_type & store.FILEFLAGS_CHANGELOG:
-        return changelog.changelog(repo.svfs)
-    elif rl_type & store.FILEFLAGS_MANIFESTLOG:
-        mandir = b''
-        if b'/' in path:
-            mandir = path.rsplit(b'/', 1)[0]
-        return manifest.manifestrevlog(
-            repo.nodeconstants, repo.svfs, tree=mandir
-        )
-    else:
-        # drop the extension and the `data/` prefix
-        path_part = path.rsplit(b'.', 1)[0].split(b'/', 1)
-        if len(path_part) < 2:
-            msg = _(b'cannot recognize revlog from filename: %s')
-            msg %= path
-            raise error.Abort(msg)
-        path = path_part[1]
-        return filelog.filelog(repo.svfs, path)
-
-
-def _copyrevlog(tr, destrepo, oldrl, rl_type, unencodedname):
+def _copyrevlog(tr, destrepo, oldrl, entry):
     """copy all relevant files for `oldrl` into `destrepo` store
 
     Files are copied "as is" without any transformation. The copy is performed
@@ -80,7 +52,7 @@ def _copyrevlog(tr, destrepo, oldrl, rl_type, unencodedname):
     content is compatible with format of the destination repository.
     """
     oldrl = getattr(oldrl, '_revlog', oldrl)
-    newrl = _revlogfrompath(destrepo, rl_type, unencodedname)
+    newrl = entry.get_revlog_instance(destrepo)
     newrl = getattr(newrl, '_revlog', newrl)
 
     oldvfs = oldrl.opener
@@ -98,7 +70,8 @@ def _copyrevlog(tr, destrepo, oldrl, rl_type, unencodedname):
     if copydata:
         util.copyfile(olddata, newdata)
 
-    if rl_type & store.FILEFLAGS_FILELOG:
+    if entry.is_filelog:
+        unencodedname = entry.main_file_path()
         destrepo.svfs.fncache.add(unencodedname)
         if copydata:
             destrepo.svfs.fncache.add(unencodedname[:-2] + b'.d')
@@ -113,18 +86,18 @@ UPGRADE_ALL_REVLOGS = frozenset(
 )
 
 
-def matchrevlog(revlogfilter, rl_type):
+def matchrevlog(revlogfilter, entry):
     """check if a revlog is selected for cloning.
 
     In other words, are there any updates which need to be done on revlog
     or it can be blindly copied.
 
     The store entry is checked against the passed filter"""
-    if rl_type & store.FILEFLAGS_CHANGELOG:
+    if entry.is_changelog:
         return UPGRADE_CHANGELOG in revlogfilter
-    elif rl_type & store.FILEFLAGS_MANIFESTLOG:
+    elif entry.is_manifestlog:
         return UPGRADE_MANIFEST in revlogfilter
-    assert rl_type & store.FILEFLAGS_FILELOG
+    assert entry.is_filelog
     return UPGRADE_FILELOGS in revlogfilter
 
 
@@ -133,19 +106,20 @@ def _perform_clone(
     dstrepo,
     tr,
     old_revlog,
-    rl_type,
-    unencoded,
+    entry,
     upgrade_op,
     sidedata_helpers,
     oncopiedrevision,
 ):
     """returns the new revlog object created"""
     newrl = None
-    if matchrevlog(upgrade_op.revlogs_to_process, rl_type):
+    revlog_path = entry.main_file_path()
+    if matchrevlog(upgrade_op.revlogs_to_process, entry):
         ui.note(
-            _(b'cloning %d revisions from %s\n') % (len(old_revlog), unencoded)
+            _(b'cloning %d revisions from %s\n')
+            % (len(old_revlog), revlog_path)
         )
-        newrl = _revlogfrompath(dstrepo, rl_type, unencoded)
+        newrl = entry.get_revlog_instance(dstrepo)
         old_revlog.clone(
             tr,
             newrl,
@@ -156,10 +130,10 @@ def _perform_clone(
         )
     else:
         msg = _(b'blindly copying %s containing %i revisions\n')
-        ui.note(msg % (unencoded, len(old_revlog)))
-        _copyrevlog(tr, dstrepo, old_revlog, rl_type, unencoded)
+        ui.note(msg % (revlog_path, len(old_revlog)))
+        _copyrevlog(tr, dstrepo, old_revlog, entry)
 
-        newrl = _revlogfrompath(dstrepo, rl_type, unencoded)
+        newrl = entry.get_revlog_instance(dstrepo)
     return newrl
 
 
@@ -200,22 +174,11 @@ def _clonerevlogs(
 
     # Perform a pass to collect metadata. This validates we can open all
     # source files and allows a unified progress bar to be displayed.
-    for rl_type, unencoded, size in alldatafiles:
-        if not rl_type & store.FILEFLAGS_REVLOG_MAIN:
+    for entry in alldatafiles:
+        if not entry.is_revlog:
             continue
 
-        # the store.walk function will wrongly pickup transaction backup and
-        # get confused. As a quick fix for 5.9 release, we ignore those.
-        # (this is not a module constants because it seems better to keep the
-        # hack together)
-        skip_undo = (
-            b'undo.backup.00changelog.i',
-            b'undo.backup.00manifest.i',
-        )
-        if unencoded in skip_undo:
-            continue
-
-        rl = _revlogfrompath(srcrepo, rl_type, unencoded)
+        rl = entry.get_revlog_instance(srcrepo)
 
         info = rl.storageinfo(
             exclusivefiles=True,
@@ -232,19 +195,19 @@ def _clonerevlogs(
         srcrawsize += rawsize
 
         # This is for the separate progress bars.
-        if rl_type & store.FILEFLAGS_CHANGELOG:
-            changelogs[unencoded] = rl_type
+        if entry.is_changelog:
+            changelogs[entry.target_id] = entry
             crevcount += len(rl)
             csrcsize += datasize
             crawsize += rawsize
-        elif rl_type & store.FILEFLAGS_MANIFESTLOG:
-            manifests[unencoded] = rl_type
+        elif entry.is_manifestlog:
+            manifests[entry.target_id] = entry
             mcount += 1
             mrevcount += len(rl)
             msrcsize += datasize
             mrawsize += rawsize
-        elif rl_type & store.FILEFLAGS_FILELOG:
-            filelogs[unencoded] = rl_type
+        elif entry.is_filelog:
+            filelogs[entry.target_id] = entry
             fcount += 1
             frevcount += len(rl)
             fsrcsize += datasize
@@ -289,16 +252,15 @@ def _clonerevlogs(
         )
     )
     progress = srcrepo.ui.makeprogress(_(b'file revisions'), total=frevcount)
-    for unencoded, rl_type in sorted(filelogs.items()):
-        oldrl = _revlogfrompath(srcrepo, rl_type, unencoded)
+    for target_id, entry in sorted(filelogs.items()):
+        oldrl = entry.get_revlog_instance(srcrepo)
 
         newrl = _perform_clone(
             ui,
             dstrepo,
             tr,
             oldrl,
-            rl_type,
-            unencoded,
+            entry,
             upgrade_op,
             sidedata_helpers,
             oncopiedrevision,
@@ -331,15 +293,14 @@ def _clonerevlogs(
     progress = srcrepo.ui.makeprogress(
         _(b'manifest revisions'), total=mrevcount
     )
-    for unencoded, rl_type in sorted(manifests.items()):
-        oldrl = _revlogfrompath(srcrepo, rl_type, unencoded)
+    for target_id, entry in sorted(manifests.items()):
+        oldrl = entry.get_revlog_instance(srcrepo)
         newrl = _perform_clone(
             ui,
             dstrepo,
             tr,
             oldrl,
-            rl_type,
-            unencoded,
+            entry,
             upgrade_op,
             sidedata_helpers,
             oncopiedrevision,
@@ -371,15 +332,14 @@ def _clonerevlogs(
     progress = srcrepo.ui.makeprogress(
         _(b'changelog revisions'), total=crevcount
     )
-    for unencoded, rl_type in sorted(changelogs.items()):
-        oldrl = _revlogfrompath(srcrepo, rl_type, unencoded)
+    for target_id, entry in sorted(changelogs.items()):
+        oldrl = entry.get_revlog_instance(srcrepo)
         newrl = _perform_clone(
             ui,
             dstrepo,
             tr,
             oldrl,
-            rl_type,
-            unencoded,
+            entry,
             upgrade_op,
             sidedata_helpers,
             oncopiedrevision,
@@ -410,7 +370,7 @@ def _files_to_copy_post_revlog_clone(srcrepo):
     are cloned"""
     for path, kind, st in sorted(srcrepo.store.vfs.readdir(b'', stat=True)):
         # don't copy revlogs as they are already cloned
-        if store.revlog_type(path) is not None:
+        if store.is_revlog_file(path):
             continue
         # Skip transaction related files.
         if path.startswith(b'undo'):
@@ -655,9 +615,14 @@ def upgrade_dirstate(ui, srcrepo, upgrade_op, old, new):
             pass
 
     assert srcrepo.dirstate._use_dirstate_v2 == (old == b'v2')
+    use_v2 = new == b'v2'
+    if use_v2:
+        # Write the requirements *before* upgrading
+        scmutil.writereporequirements(srcrepo, upgrade_op.new_requirements)
+
     srcrepo.dirstate._map.preload()
-    srcrepo.dirstate._use_dirstate_v2 = new == b'v2'
-    srcrepo.dirstate._map._use_dirstate_v2 = srcrepo.dirstate._use_dirstate_v2
+    srcrepo.dirstate._use_dirstate_v2 = use_v2
+    srcrepo.dirstate._map._use_dirstate_v2 = use_v2
     srcrepo.dirstate._dirty = True
     try:
         srcrepo.vfs.unlink(b'dirstate')
@@ -667,8 +632,9 @@ def upgrade_dirstate(ui, srcrepo, upgrade_op, old, new):
         pass
 
     srcrepo.dirstate.write(None)
-
-    scmutil.writereporequirements(srcrepo, upgrade_op.new_requirements)
+    if not use_v2:
+        # Remove the v2 requirement *after* downgrading
+        scmutil.writereporequirements(srcrepo, upgrade_op.new_requirements)
 
 
 def upgrade_tracked_hint(ui, srcrepo, upgrade_op, add):
